@@ -2,6 +2,9 @@
 
 #include <iconv.h>
 
+extern void
+ptp_debug (PTPParams *params, const char *format, ...);
+
 static inline uint16_t
 htod16p (PTPParams *params, uint16_t var)
 {
@@ -992,10 +995,20 @@ ptp_unpack_OPL (PTPParams *params, unsigned char* data, MTPProperties **pprops,
 		*pprops = NULL;
 		return 0;
 	}
+	ptp_debug (params ,"Unpacking MTP OPL, size %d (prop_count %d)", len, prop_count);
 	data += sizeof(uint32_t);
+	len -= sizeof(uint32_t);
 	props = malloc(prop_count * sizeof(MTPProperties));
 	if (!props) return 0;
 	for (i = 0; i < prop_count; i++) {
+		if (len <= 0) {
+			ptp_debug (params ,"short MTP Object Property List at property %d (of %d)", i, prop_count);
+			ptp_debug (params ,"device probably needs DEVICE_FLAG_BROKEN_MTPGETOBJPROPLIST_ALL", i);
+			ptp_debug (params ,"or even DEVICE_FLAG_BROKEN_MTPGETOBJPROPLIST", i);
+			qsort (props, i, sizeof(MTPProperties),_compare_func);
+			*pprops = props;
+			return i;
+		}
 		props[i].ObjectHandle = dtoh32a(data);
 		data += sizeof(uint32_t);
 		len -= sizeof(uint32_t);
@@ -1098,9 +1111,6 @@ ptp_unpack_Canon_FE (PTPParams *params, unsigned char* data, PTPCANONFolderEntry
 #define PTP_ece_OI_Size		0x14	/* only for objectinfos */
 #define PTP_ece_OI_Name		0x1c	/* only for objectinfos */
 
-extern void
-ptp_debug (PTPParams *params, const char *format, ...);
-
 static inline int
 ptp_unpack_CANON_changes (PTPParams *params, unsigned char* data, int datasize, PTPCanon_changes_entry **ce)
 {
@@ -1145,7 +1155,7 @@ ptp_unpack_CANON_changes (PTPParams *params, unsigned char* data, int datasize,
 			int		j;
 			PTPDevicePropDesc	*dpd;
 
-			ptp_debug (params, "event %d: EOS prop %04x desc record, datasize %d", i, proptype, size-PTP_ece_Prop_Desc_Data);
+			ptp_debug (params, "event %d: EOS prop %04x desc record, datasize %d, propxtype %d", i, proptype, size-PTP_ece_Prop_Desc_Data, propxtype);
 			for (j=0;j<params->nrofcanon_props;j++)
 				if (params->canon_props[j].proptype == proptype)
 					break;
@@ -1154,6 +1164,10 @@ ptp_unpack_CANON_changes (PTPParams *params, unsigned char* data, int datasize,
 				break;
 			}
 			dpd = &params->canon_props[j].dpd;
+			/* 1 - uint16 ? 
+			 * 3 - uint16
+			 * 7 - string?
+			 */
 			if (propxtype != 3) {
 				ptp_debug (params, "event %d: propxtype is %x for %04x, unhandled.", i, propxtype, proptype);
 				for (j=0;j<size-PTP_ece_Prop_Desc_Data;j++)
@@ -1166,6 +1180,10 @@ ptp_unpack_CANON_changes (PTPParams *params, unsigned char* data, int datasize,
 				dpd->FORM.Enum.SupportedValue = malloc (sizeof (PTPPropertyValue)*propxcnt);
 				for (j=0;j<propxcnt;j++) {
 					switch (dpd->DataType) {
+					case PTP_DTC_INT16:
+						dpd->FORM.Enum.SupportedValue[j].i16	= dtoh16a(data);
+						ptp_debug (params, "event %d: suppval[%d] of %x is %x.", i, j, proptype, dtoh16a(data));
+						break;
 					case PTP_DTC_UINT16:
 						dpd->FORM.Enum.SupportedValue[j].u16	= dtoh16a(data);
 						ptp_debug (params, "event %d: suppval[%d] of %x is %x.", i, j, proptype, dtoh16a(data));
@@ -1227,17 +1245,48 @@ ptp_unpack_CANON_changes (PTPParams *params, unsigned char* data, int datasize,
 				case PTP_DPC_CANON_EOS_Aperture:
 				case PTP_DPC_CANON_EOS_ShutterSpeed:
 				case PTP_DPC_CANON_EOS_ISOSpeed:
+				case PTP_DPC_CANON_EOS_Focus:
+				case PTP_DPC_CANON_EOS_CameraMode:
+				case PTP_DPC_CANON_EOS_ColorSpace:
+				case PTP_DPC_CANON_EOS_TransferOption:
 					dpd->DataType = PTP_DTC_UINT16;
 					break;
 				case PTP_DPC_CANON_EOS_PictureStyle:
 				case PTP_DPC_CANON_EOS_WhiteBalance:
 				case PTP_DPC_CANON_EOS_MeteringMode:
-				case PTP_DPC_CANON_EOS_ExpCompensation:
+				case PTP_DPC_CANON_EOS_ExpCompensation: /* actually int8 if you calculate */
 					dpd->DataType = PTP_DTC_UINT8;
 					break;
 				case PTP_DPC_CANON_EOS_Owner:
 					dpd->DataType = PTP_DTC_STR;
 					break;
+				case PTP_DPC_CANON_EOS_WhiteBalanceAdjustA:
+				case PTP_DPC_CANON_EOS_WhiteBalanceAdjustB:
+					dpd->DataType = PTP_DTC_INT16;
+					break;
+				/* unknown props, listed from dump.... all 16 bit, but vals might be smaller */
+				case 0xd106:
+				case 0xd10e:
+				case 0xd112:
+				case 0xd114:
+				case 0xd119:
+				case 0xd11a:
+				case 0xd11b:
+				case 0xd11c:
+				case 0xd11d:
+				case 0xd180:
+				case 0xd181:
+				case 0xd182:
+				case 0xd183:
+				case 0xd184:
+				case 0xd185:
+				case 0xd186:
+				case 0xd187:
+				case 0xd188:
+				case 0xd189:
+				case 0xd18a:
+					dpd->DataType = PTP_DTC_UINT16;
+					break;
 				default:
 					ptp_debug (params, "event %d: Unknown EOS property %04x, datasize is %d", i ,proptype, size-PTP_ece_Prop_Val_Data);
 					for (j=0;j<size-PTP_ece_Prop_Val_Data;j++)
@@ -1264,7 +1313,7 @@ ptp_unpack_CANON_changes (PTPParams *params, unsigned char* data, int datasize,
 					uint8_t len = 0;
 					dpd->FactoryDefaultValue.str	= ptp_unpack_string(params, data, 0, &len);
 					dpd->CurrentValue.str		= ptp_unpack_string(params, data, 0, &len);
-					ptp_debug (params,"event %d: currentvalue of %x is %s", i, proptype, dpd->CurrentValue.u8);
+					ptp_debug (params,"event %d: currentvalue of %x is %s", i, proptype, dpd->CurrentValue.str);
 					break;
 				}
 				default:
@@ -1273,6 +1322,11 @@ ptp_unpack_CANON_changes (PTPParams *params, unsigned char* data, int datasize,
 				}
 				break;
 		}
+		case 0: /* end marker */
+			if (size == 8) /* no output */
+				break;
+			ptp_debug (params, "event %d: EOS property type 0, but size %d", i, size);
+			break;
 		default:
 			ptp_debug (params, "event %d: unknown EOS property type %04x", i, type);
 			(*ce)[i].type = PTP_CANON_EOS_CHANGES_TYPE_UNKNOWN;
